#!/bin/sh
exec $ZEUSHOME/perl/miniperl -x $0 ${1+"$@"}
 if 0;
#!/usr/bin/perl -w

# /apps/admin/certs/cert.cgi and /apps/web/sslwiz.cgi are identical

BEGIN { @INC=( "$ENV{ZEUSHOME}/perl/", "$ENV{ZEUSHOME}/webadmin/etc/" ); }
use ZeusOS;
use WebAdmin;

# First off, ensure this page isn't cached, otherwise it can be very
# confusing!
$| = 1;

print <<EOM;
Pragma: no-cache\r
Cache-Control: no-cache\r
Expires: 0\r
\r
<body bgcolor=white>
EOM

&parse_form_data(*arg);

# We're expecting the following args:
#   _base, _redirect
#   private_key, public_key, cert_request
#   dns, org, orgu, loc, state, country

$_base        = $arg{_base}        || "";
$_redirect    = $arg{_redirect}    || "";
$private_key  = $arg{_add_text_private_key}  
                                   || FATAL("No private key file specified");
$key_size     = $arg{key_size}     || "1024";
$public_cert  = $arg{_add_text_public_key}  
                                   || FATAL("No public cert. file specified");
$cert_request = $arg{cert_request} || "";

$id_dns       = $arg{dns}     || FATAL("No 'dns' value supplied");
$id_org       = $arg{org}     || FATAL("No 'Organisation' value supplied");
$id_orgu      = $arg{orgu}    || ".";
$id_loc       = $arg{loc}     || FATAL("No 'location' value supplied");
$id_state     = $arg{state}   || ".";
$id_country   = $arg{country} || FATAL("No 'country' value supplied");

$verbose      = $arg{verbose} || "";

$private_key =~ /^\// or 
   FATAL("private key must be an absolute file name (beginning '/')");
$public_cert =~ /^\// or 
   FATAL("public certificate must be an absolute file name (beginning '/')");

print "<dl>\n";

print "<dt><h3>Generating private key</h3>\n<dd>\n";

if( -f $private_key ) {
   $/ = "\n";
   open KEY, "<$private_key";
   if (<KEY> !~ /^-----BEGIN RSA PRIVATE KEY-----\n$/) {
      FATAL("$private_key exists and is not a valid private key\n");
   }
   close KEY;
   print "<b>$private_key</b> already exists; using this file\n";
} else {
   $cmd = "\$ZEUSHOME/admin/bin/cert -new -type private -keysize $key_size -out $private_key";
   
   $verbose and print "<pre><b>$cmd</b>\n\n";

   $/ = "\n";
   open CMD, "$cmd 2>&1 |";
   while (<CMD>) { $verbose and print "   $_"; }
   $verbose and print "</pre>\n";

   close CMD;
   $? and FATAL("</dl>Command exited abnormally - aborting\n");
   
   print "Your new private key has been written to '$private_key'\n";
}

print "<dt><h3>Generating public certificate</h3>\n<dd>\n";

if( -f $public_cert ) {
   $/ = "\n";
   open KEY, "<$public_cert";
   if (<KEY> !~ /^-----BEGIN CERTIFICATE-----$/) {
      FATAL("$public_cert exists and is not a valid public certificate\n");
   }
   close KEY;
   print "<b>$public_cert</b> already exists; using this file\n";
} else {
   $cmd = "\$ZEUSHOME/admin/bin/cert -new -type public -key $private_key -out $public_cert";
   
   $verbose and print "<pre><b>$cmd</b>\n";
   open CMD, "|$cmd 2>&1 > /tmp/cert.$$";
   
   print CMD "$id_country\n$id_state\n$id_loc\n$id_org\n$id_orgu\n$id_dns\n";
   
   close CMD;
   $? and FATAL("</dl>Command exited abnormally - aborting\n");

   undef $/;
   open IN, "</tmp/cert.$$";
   $txt = "   ".<IN>;
   close IN;
   unlink "/tmp/cert.$$";
   
   $txt =~ s/:  /: <b>$id_country<\/b>\n/;
   $txt =~ s/:  /: <b>$id_state<\/b>\n/;
   $txt =~ s/:  /: <b>$id_loc<\/b>\n/;
   $txt =~ s/:  /: <b>$id_org<\/b>\n/;
   $txt =~ s/:  /: <b>$id_orgu<\/b>\n/;
   $txt =~ s/:  /: <b>$id_dns<\/b>\n/;
   $txt =~ s/\n/\n   /g; #indent
   $verbose and print "$txt</pre>\n";

   print "Your new public certificate has been written to the file '$public_cert'.\n";
}

if( $verbose ) {
   print "<dt><h3>Displaying public certificate</h3>\n<dd>\n";

   $cmd = "\$ZEUSHOME/admin/bin/cert -in $public_cert -text";
   
   print "<pre><b>$cmd</b>\n\n";
   
   $/ = "\n";
   open CMD, "$cmd 2>&1 |";
   
   while (<CMD>) { print "   $_"; }
   print "</pre>\n";
   
   close CMD;
   $? and FATAL("</dl>Command exited abnormally - aborting\n");
}

print "<dt><h3>Verifying Key/Certificate pair</h3>\n<dd>\n";

$cmd = "\$ZEUSHOME/admin/bin/cert -check -key $private_key -in $public_cert";

$verbose and print "<pre><b>$cmd</b>\n\n";

$/ = "\n";
open CMD, "$cmd 2>&1 |";

while (<CMD>) { $verbose and print "   $_"; }
$verbose and print "</pre>\n";

close CMD;
$? and FATAL("</dl>Command exited abnormally - aborting\n");

print "Done\n";

if ( $cert_request ) {
   print "<dt><h3>Generating certificate request</h3>\n<dd>\n";
   
   if( -f $cert_request ) {
      $/ = "\n";
      open KEY, "<$cert_request";
      if (<KEY> !~ /^-----BEGIN NEW CERTIFICATE REQUEST-----\n$/) {
         FATAL("$cert_request exists and is not a valid certificate request\n");
      }
      close KEY;
      print "<b>$cert_request</b> already exists; using this file\n";
   } else {
      $cmd = "\$ZEUSHOME/admin/bin/cert -new -type request -key $private_key -out $cert_request";
      
      $verbose and print "<pre><b>$cmd</b>\n";
      open CMD, "|$cmd 2>&1 > /tmp/cert.$$";
      
      print CMD "$id_country\n$id_state\n$id_loc\n$id_org\n$id_orgu\n$id_dns\n";
      
      close CMD;
      $? and FATAL("</dl>Command exited abnormally - aborting\n");
   
      undef $/;
      open IN, "</tmp/cert.$$";
      $txt = "   ".<IN>;
      close IN;
      unlink "/tmp/cert.$$";
      
      $txt =~ s/:  /: <b>$id_country<\/b>\n/;
      $txt =~ s/:  /: <b>$id_state<\/b>\n/;
      $txt =~ s/:  /: <b>$id_loc<\/b>\n/;
      $txt =~ s/:  /: <b>$id_org<\/b>\n/;
      $txt =~ s/:  /: <b>$id_orgu<\/b>\n/;
      $txt =~ s/:  /: <b>$id_dns<\/b>\n/;
      $txt =~ s/\n/\n   /g; #indent
      $verbose and print "$txt</pre>\n";
   
      print "Your certificate request has been written to the file '$cert_request'.\n";
   }
   
   if ($verbose) {
      print "<dt><h3>Displaying certificate request</h3>\n<dd>\n";
   
      $cmd = "\$ZEUSHOME/admin/bin/cert -in $cert_request -text";
      
      print "<pre><b>$cmd</b>\n\n";
      
      $/ = "\n";
      open CMD, "$cmd 2>&1 |";
      while (<CMD>) { print "   $_"; }
      print "</pre>\n";
      close CMD;
      $? and FATAL("</dl>Command exited abnormally - aborting\n");
   }
}

if ( $_base ) {
   # Commit changes
   print "<dt><h3>Updating Website</h3>\n<dd>\n";
   $/ = "\n";
   open CMD, "$ENV{ZEUSHOME}/admin/docroot/apps/admin/regupdate.cgi 2>&1 |";
   while (<CMD>) { last if /^\r?\n$/; }
   while (<CMD>) { print "<tt>   </tt>$_"; }
   close CMD;
   $? and FATAL("</dl>Command exited abnormally - aborting\n");

   print "Done</dl>\n";
}

print footer();
exit 0;

sub FATAL {
   local $msg = shift;
   print "<b>Error</b>: $msg\n";
   print footer();
   exit 0;
}

sub footer {
   if( $_redirect) {
      return "<p><a href=\"$_redirect\"><img src=\"back.gif\" border=0></a>\n";
   } else {
      return "\n";
   }
}

sub parse_form_data {
   local (*FORM_DATA) = @_;

   local ($request_method, $query_string, $key_value, $key, $value);

   $request_method = $ENV{'REQUEST_METHOD'};

   if ($request_method eq "GET") {
      $query_string = $ENV{'QUERY_STRING'};
   } elsif ($request_method eq "POST") {
      $query_string = "";
      read STDIN, $query_string, $ENV{'CONTENT_LENGTH'};
   }

   foreach $key_value (split /&/, $query_string) {
      ($key, $value) = split /=/, $key_value, 2;
      $value =~ tr/+/ /;
      $value =~ s/%([\dA-Fa-f][\dA-Fa-f])/pack("C", hex($1))/eg;
      if (defined $FORM_DATA{$key}) {
         $FORM_DATA{$key} = join " ", $FORM_DATA{$key}, $value;
      } else {
         $FORM_DATA{$key} = $value;
      }
   }
   return 0;
}              
