<html>
<head>
<title>Cluster status</title>
<% 
response::setheader "Pragma" "no-cache"
response::setheader "Expires" "0"

#############################################################################
# Check for auto-refresh
global refresh
set refresh 0
asp::decode_input
if {[info exists asp::input(refresh)] && $asp::input(refresh)=="yes" } {
    set refresh 1
    asp::write "<meta http-equiv=Refresh content=\"3; $request::env(SCRIPT_NAME)?refresh=yes\">"
}

#############################################################################
## Attempts to connect with a timeout, either errors or returns a 
## socket channel (blocking)
##
proc connect_timeout { host port timeout } {
    global connect_timeout ;# global table for our state info
    set connect_timeout(connected) ""
    set connect_timeout(socket) [socket -async $host $port]
    set t [expr $timeout*1000]
    set connect_timeout(timeoutid) [after $t { 
	close $connect_timeout(socket)
	set connect_timeout(connected) "no"
    }]
    fileevent $connect_timeout(socket) writable { 
	after cancel $connect_timeout(timeoutid)
	set connect_timeout(connected) "yes"
    }
    # Now wait if we didn't manage it straight away....
    if { $connect_timeout(connected) == "" } { 
	vwait connect_timeout(connected) 
    }
    if { $connect_timeout(connected) == "yes" } { 
	# Ensure we are back blocking again
	fconfigure $connect_timeout(socket) -blocking yes 
	return $connect_timeout(socket) 
    }
    error "Connect timeout"
}

#############################################################################
##
## Draws a pictorial representation of the current server status
##
## Requries
##    zap functionality
##    single-black.gif
##    server.gif
##    deadserver.gif

proc draw_status {} {

    global refresh

    #
    ## Registry settings
    #
    # List of backend machines (host:port)
    set hostsfile "$request::env(ZEUSHOME)/webadmin/conf/hosts"
    # List of frontend load-balancers (host:port)
    set lbsfile   "$request::env(ZEUSHOME)/webadmin/conf/lbs"

    # Get current time (used to generate unique URLs for machines)
    set count [ clock seconds ]

    # ----------------------------------------------------------
    # Returns the response time in ms
    proc getstats { backend } {
	# TODO
	return "";
    }
    
    # ----------------------------------------------------------
    # Returns the hostname of a machine given its full name
    
    proc hostname { fullname } {
        set hostonly [ lindex [ split $fullname : ] 0 ]

        # Check if this hostname is just an IP address
        if { [ regexp \[0-9\]+\.\[0-9\]+\.\[0-9\]+\.\[0-9\]+ $hostonly ] } {
            return $hostonly
        } else {
            return [lindex [split $hostonly  .] 0]
        }
    }
    
    # ----------------------------------------------------------
    
    ## Get the settings
    set f [open $hostsfile]
    set backends [lsort -dictionary [read $f]]
    close $f

    # this may not exist if Zeus Balancer has not been installed, so we
    # have to be slightly careful, hence the catch.
    set frontends {}
    catch {
	set f [open $lbsfile]
	set frontends [lsort -dictionary [read $f]]
	close $f
    }

    # If there are any frontends, there should be 1 or 2 of them
    #
    if { [llength $frontends] > 2 } {
        asp::write "<h3>Warning: [llength $frontends] frontends defined. "
        asp::write "There is a maximum of 2!</h3>"
        set frontends {}
    }

    if { $refresh } {
        set lowsrc ""
    } else {
        set lowsrc "lowsrc=\"timeserver.gif\""
    }
    
    ## Now lets think about our table
    #
    # |-------------------------------------------------|
    # |backend1|...|backend2|....|backend3|....|backend4|
    # ---------------------------------------------------

    # or like this:
    # |-------------------------------------------------|
    # |..........|frontend1|........|frontend2|.........|
    # |-------------------------------------------------|
    # |backend1|...|backend2|....|backend3|....|backend4|
    # ---------------------------------------------------
    
    # Work out how many columns we need
    set columns [llength $frontends]

    # Start our table
    asp::write "<center><table border=0 cellspacing=0 cellpadding=0>\n"
    
    # We need a router!
    asp::write "<tr><td colspan=$columns align=middle>
    <img src=router.gif><br>
    <img src=single-black.gif height=20 width=1></td></tr>\n"

    # Now the bus onto frontend connections (if there is a frontend)
    if { [llength $frontends] } {

        # Our bus
        asp::write "<tr bgcolor=#000000><td height=1 colspan=$columns>
        <img src=single-black.gif width=1 height=1></td></tr>\n"

	# Write our frontend row
	asp::write "<tr>"
	foreach i $frontends {
	    asp::write "<td align=middle valign=top>
	    <a href=\"lb.cgi?$i\"><img $lowsrc src=checkserver.cgi?$i:no:$count width=66 height=106 hspace=30 border=0><br><font size=-1>[hostname $i]</font></a></td>"
	}
	asp::write "</tr>\n"

	# Now our frontend onto bus row
	asp::write "<tr>"
	foreach i $frontends {
	    asp::write "<td align=middle><img src=single-black.gif height=20 width=1></td>"
	}
	asp::write "</tr>\n"

    }

    asp::write "</table>\n"
    set image_width 66
    set image_height 106
    set small no
    set max_width 440
    set first_row 1

    while { [ llength $backends ] > 0 } {

        # Start a new table, so we can space out backends separately
        asp::write "<table border=0 cellspacing=0 cellpadding=0>"

        set num_backends [ llength $backends ]
        set spacing [ expr ( $max_width / $num_backends ) ]
    
        if { $spacing < $image_width + 6 } {
            set small yes
            set image_width 33
            set image_height 53
            
            if { $lowsrc != "" } {
                set lowsrc "lowsrc=\"mini-timeserver.gif\""
            }

            set spacing [ expr ( $max_width / $num_backends ) ]
            if { $spacing < $image_width + 12 } {
                set spacing [ expr $image_width + 12 ]
            }
        }
        if { $spacing > $image_width + 60 } {
            set spacing [ expr $image_width + 60 ]
        }

        # How many servers can fit onto a row?
        set per_row [ expr $max_width / $spacing ]
        set i [ expr $per_row + 1 ]

        if { $first_row && $per_row >= $num_backends } {
            asp::write "<tr bgcolor=#000000><td height=1 colspan=$num_backends>
            <img src=single-black.gif height=1></td></tr>\n"
        } else {
            asp::write "<tr bgcolor=#000000><td height=1 colspan=$i>
            <img src=single-black.gif width=$max_width height=1></td></tr>\n"
        }

        # Now the backend onto bus row
        asp::write "<tr>"
            
        if { $per_row >= $num_backends } {
            set per_row $num_backends
        } else {
            asp::write "<td align=middle valign=top bgcolor=#000000 width=1><img src=single-black.gif></td>"
        }

        for { set i 0 } { $i < $per_row } { incr i } {
            set box [ lindex $backends 0 ]
            set backends [ lrange $backends 1 end ]
            asp::write "<td align=middle valign=top width=$spacing>
	    <a href=\"machine.zap?machine=$box\"><img $lowsrc src=checkserver.cgi?$box:$small:$count width=$image_width height=$image_height border=0><br><font size=-1>[hostname $box]</font></a><br><br></td>"
        }

        # All done
        asp::write "</tr>\n</table>\n"

        set first_row 0
    }

    asp::write "</center>"

}
##############################################################################

%>
</head>

<body bgcolor="#ffffff" link=#990066 vlink=#990066 alink=#990066>

<table>
<tr>
 <td valign=top>
  <img src="/apps/web/title.gif" hspace=10 alt="">
  <p align=right>

  <a href="/apps/admin/docsystem/context.asp?help=/apps/web/docs/controller/cluster.html" target="help" onClick=
     "window.open('','help','resizable=1,width=640,height=500'); return true;">
        <img src="/apps/web/docs.gif" 
        alt="[documentation]" align=bottom border=0 hspace=0></a>

  <a href="/apps/web/"><img src="/apps/web/back.gif" 
        alt="[back]" align=bottom border=0 hspace=0></a>
        <a href="/apps/web/"><img src="/apps/web/home.gif" alt="[Controller]" 
        align=bottom border=0></a>
  </td>
  <td valign=top>


<br>
<img src="title-clusterstatus.gif" alt="Cluster Status">

<p>

Zeus Web Server has been designed to easily manage a cluster of
machines with a common configuration.  This page lets you view and
configure your cluster setup. Click on a server icon to view the 
status of the server. Click on the book icon for more information
about setting up a cluster.

<p><center>
<%
if { $refresh } {
    asp::write "<a href=\"$request::env(SCRIPT_NAME)\">
    Disable auto-refresh</a>"
} else {
    asp::write "<a href=\"$request::env(SCRIPT_NAME)?refresh=yes\">
    Enable auto-refresh</a>"
}
%>
&nbsp;&nbsp;&nbsp;&nbsp;
<a href="settings.zap">Configuration</a>
&nbsp;&nbsp;&nbsp;&nbsp;
<a href="summary.cgi">Summary</a>
</center>
<ul>
<p>
<%
draw_status
%>
</ul>

</td>
</tr>
</table>

</body>
</html>
